%%% Determines the stability of the beam based off of the numerical
%%% correlation between amplitude and DY (change in distance between tip of
%%% cone and some reference point).

%%% workflow order: trackDisplacement3a.m (get raw position data) ->
%%% plotDY.m (post-process, finds velocity and buckle criteria based on
%%% numerical findings)
close all; clear all; clc;
cd DATA
ind = 1;
cc = 'br'; %colors
for L = [5 8];
    clearvars -except L ind cc
A = 60;
H = 2;
ot = 1; %other filename (1 or 2)
%length =[1    2    3    4     5     6     7     8      9      10]
buckle = [7.4 3.56 2.355 1.76 1.408 1.177 1.0295 0.884 0.7785  0.684]; %theoretical buckle lines (deltaY)
thickness = 3.5; %mm

lowerBound = -2; %bounds for stability regions (graphical purpose only)
upperBound = 10; %bounds for stability regions (graphical purpose only)
nh = 2; % number of cone heights below surface to analyze

if ot == 1
    name = ['A' int2str(A) '_CG_H' int2str(H) '_L' int2str(L) ''];
elseif ot==2
    name = ['A' int2str(A) '_CG_H' int2str(H) '_L' int2str(L) '_1'];
elseif ot==3
    name = ['A' int2str(A) '_CG_H' int2str(H) '_L' int2str(L) '_2'];
end
load([name '.mat'],'coneX','coneY','refX','refY','pix_m','pix_m2','time','dtImp','dtSub','fps');
load(['fitCoef_L' num2str(L) '.mat'])

nt = size(coneX,2); % number of trials
beta = (A/2)*(pi/180); %%% cone HALF angle, radians
Dt = round(nh*(dtSub-dtImp)); %% time of nr radius below surface

for i = 1:nt
    L0_pix(i) = mean(abs(refY(1,i) - coneY(1,i))); %in pixels, initial distance from ref to cone
    interface = coneY(1,i); % location of interface
    L1_pix(:,i) = interface - refY(:,i); % distance from ref point to interface
    L2_pix(:,i) = interface - coneY(:,i); % distance from cone tip to interface
end
L0 = L0_pix./pix_m; % in meter, initial distance from ref to cone
L1 = L1_pix./pix_m; % in meter, distance from ref point to interface, almost always above water surface

%%%%%% in meter, distance from cone tip to interface %%%%%%%
for i = 1:nt % trial steps
%     f = nnz(time(:,i));
%     t1 = time(1:f,i);
for j = 1:length(time) % time steps
    if j<dtImp(i)
        L2(j,i) = L2_pix(j,i)./pix_m; % above water calibration
        Y(j,i) = coneY(j,i)./pix_m;
    elseif j >= dtImp(i)
        L2(j,i) = L2_pix(j,i)./pix_m2; % under water calibration
        Y(j,i) = coneY(j,i)./pix_m2;
    end
end
end
d = L1-L2;


for i = 1:nt
    for j = 1:length(d)
        dy(:,i) = L0(i)-d(:,i); %change in vertical length, meters
        
        if dy(j,i)*1000<dyCrit
            dx(j,i) = 0; %mm
        else
            dx(j,i) = polyval(p,dy(j,i)*1000-dyCrit); %mm
        end
    end
end

col0 = autumn;
col1 = summer;
% find(col0(:,1)==1)
x = linspace(0,150,10);
y1 = ones(10,1)*buckle(L); % buckle line

%%%%%%%%%%%%%%%%%% PLOTS %%%%%%%%%%%%%%%%%%%%%%
% unstable = area(x,y1,upperBound); hold on;
% stable = area(x,y1,lowerBound);
% set(stable,'FaceColor',col1(45,:)); % color of stable region
% set(unstable,'FaceColor',col0(29,:)); %color of unstable region


if ot == 1 || ot == 2
dtImp = dtImp(1)*ones(10,1);
dtSub = dtSub(1)*ones(10,1);
Dt_plot = (Dt(1)+dtImp(1))*ones(10,1);
elseif ot == 3
    dtImp = dtImp(4)*ones(10,1);
dtSub = dtSub(4)*ones(10,1);
Dt_plot = (Dt(4)+dtImp(4))*ones(10,1);
end

figure(101)
subplot(1,2,1)
yy = linspace(-10,10,10);
plot(time*1000,dy*1000,'-o'); hold on; %plot in mm
plot(x,y1,'-k','linewidth',3); % plot buckle line

plot(Dt_plot(1)*ones(1,nt),dy(Dt_plot(1),:)*1000,'o','linewidth',5)
rr = y1(1) - dy(Dt_plot(1),:)*1000; %buckleLine-dy
%if qq>0, stable; if qq<0 unstable;
disp('0: stable (non-buckle); 1: unstable (buckle)');
qq = rr'<0
if sum(qq)==0
    str = 'System is stable';
    disp(str)
elseif sum(qq)==numel(qq)
    str = 'System is unstable';
    disp(str)
else
    str = 'Transitionally unstable';
    disp(str)
end
plot(dtImp,yy,'-k'); %plot vertical line where impact begins
plot(dtSub,yy,'-k'); %plot vertical line where submerged begins
plot(Dt_plot,yy,'-k'); %plot vertical line whern cone reaches nr radius below surface

xlabel('Time, ms'); ylabel('\Deltay, mm');
ylim([lowerBound upperBound])
str1 = [num2str(nh) ' cone heights below surface'];
str2 = 'Impact';
str3 = 'Submerged';
poitxt = text(Dt(1)+dtImp(1)-3,3,str1); set(poitxt,'rotation',90);
imptxt = text(dtImp(1)-3,6,str2); set(imptxt,'rotation',90);
subtxt = text(dtSub(1)-3,6,str3); set(subtxt,'rotation',90);

title(str)
% legend('Unstable Region','Stable Region','Trial 1','Trial 2','Trial 3','Trial 4','Trial 5')
xlim([0 70])

disp([ 'DY/L: ' ])
disp([ num2str(dy(Dt_plot(1),:)'./(L/100) ) ])

figure(101)
subplot(1,2,2)
plot(time*1000,dx,'-o'); hold on;
plot(dtImp,yy,'-k'); %plot vertical line where impact begins
plot(dtSub,yy,'-k'); %plot vertical line where submerged begins
plot(Dt_plot,yy,'-k'); %plot vertical line whern cone reaches nr radius below surface
xlin = linspace(0,100,100); ylin = thickness*ones(100,1);
plot(xlin,ylin,'k-','linewidth',3)
poitxt = text(Dt(1)+dtImp(1)-3,3,str1); set(poitxt,'rotation',90);
imptxt = text(dtImp(1)-3,6,str2); set(imptxt,'rotation',90);
subtxt = text(dtSub(1)-3,6,str3); set(subtxt,'rotation',90);

ylim([-2 20]); xlim([0 70]); 
xlabel('Time, ms'); ylabel('\Deltax, mm')



%%%%%% manuscript figures %%%%%%%%%%%%

figure(102)
time0 = time*1000-dtImp(1);
for i = 1:size(dx,2)
DX(:,i) = smooth(dx(:,i),'moving')/thickness;
end
Tscale = dtSub(1)-dtImp(1);
adjImp = (dtImp-dtImp(1))/Tscale;
adjSub = (dtSub-dtImp(1))/Tscale;
adj2H = (Dt_plot-dtImp(1))/Tscale;
avgDX = mean(DX,2);
stdDX = std(DX,0,2);
e = std(DX,0,2);

t0 = time0/Tscale; %scaled time
 hold on;
xlin = linspace(0,3,100); ylin = 1*ones(100,1);


uplim = avgDX+stdDX;
lowlim= avgDX-stdDX;
% plot(t1,uplim,'-')
% plot(t1,lowlim,'-')

X = [t0, fliplr(t0)];
Y = [lowlim', fliplr(uplim')];
p = fill(X,Y,'b','FaceColor',0.5*[1 1 1],'FaceAlpha',0.5,'EdgeAlpha',0);
% plot(t0,DX,'-o');
plot(t0,avgDX,['-' cc(ind)],'linewidth',3) %%%% Mean line
plot(xlin,ylin,'k-','linewidth',2)
xlim([0 2]); ylim([0 3])

plot(adjImp,avgDX(dtImp(1)),'ko','MarkerSize',5,'markerfacecolor','k'); %plot vertical line where impact begins
plot(adjSub,avgDX(dtSub(1)),'ko','MarkerSize',5,'markerfacecolor','k'); %plot vertical line where submerged begins
plot(adj2H,avgDX(Dt_plot),'ko','MarkerSize',5,'markerfacecolor','k'); %plot vertical line whern cone reaches nr radius below surface
% figure(103)
% errorbar(avgDX,e,'rx')
xlabel('Nondimensional Time'); ylabel('Nondimensional Amplitude');
% title(['Angle ' num2str(A) ', Length ' num2str(L) ', Height ' num2str(H)])
box on;
daspect([2 7 1])
ind = ind+1;
end











